==> acs_0.25.orig.tar.gz: acs-0.25/src/s__map.cc <==
/*$Id: s__map.cc,v 14.12 1999/07/23 10:13:54 al Exp $ -*- C++ -*-
 * Allocates space for the admittance matrix.  Allocation below the
 * diagonal is by row, above the diagonal is by column, and stored backwards.
 * This broken vector is stored.  The length of it increases with increasing
 * position.  The maximum length of the nth vector is 2n-1.  For a band matrix
 * only those elements that are non-zero or are nearer to the diagonal than a
 * non-zero element are stored.
 */
#include "e_card.h"
#include "u_status.h"
#include "s__.h"
/*--------------------------------------------------------------------------*/
//static void	 SIM::map_nodes();
//static void	 SIM::order_reverse();
//static void	 SIM::order_forward();
//static void	 SIM::order_auto();
//static void	 SIM::map_list(CARD*);
/*--------------------------------------------------------------------------*/
/* map_nodes: map intermediate node number to internal node number.
 * Ideally, this function would find some near-optimal order
 * and squash out gaps.
 */
/*static*/ void SIM::map_nodes()
{
  nm = new int[STATUS::total_nodes+1];
  STATUS::order.reset().start();
  switch (OPT::order){
    default:
      error(bWARNING, "invalid order spec: %d\n", OPT::order);
    case oAUTO:	   order_auto();    break;
    case oREVERSE: order_reverse(); break;
    case oFORWARD: order_forward(); break;
  }
  STATUS::order.stop();
  map_list(CARD_LIST::card_list);
}
/*--------------------------------------------------------------------------*/
/* order_reverse: force ordering to reverse of user ordering
 *  subcircuits at beginning, results on border at the bottom
 */
/*static*/ void SIM::order_reverse()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = STATUS::total_nodes - node + 1;
}
/*--------------------------------------------------------------------------*/
/* order_forward: use user ordering, with subcircuits added to end
 * results in border at the top (worst possible if lots of subcircuits)
 */
/*static*/ void SIM::order_forward()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = node;
}
/*--------------------------------------------------------------------------*/
/* order_auto: full automatic ordering
 * reverse, for now
 */
/*static*/ void SIM::order_auto()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = STATUS::total_nodes - node + 1;
}
/*--------------------------------------------------------------------------*/
/* map_list: map intermediate node to internal (working) node number.
 * recursive: actual mapping done here
 */
/*static*/ void SIM::map_list(CARD_LIST& cl)
{
  for (CARD_LIST::iterator  ci = cl.begin();  ci != cl.end();  ++ci){
    CARD* brh = *ci;
    if (brh->is_device()){
      int ii;
      for (ii = 0;  brh->n[ii].t != INVALIDNODE;  ii++){
	brh->n[ii].m = nm[brh->n[ii].t];
      }
      brh->n[ii].m = INVALIDNODE;
      if (brh->subckt().exists()){
	map_list(brh->subckt());
      }
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

==> acs-0.28.tar.gz: acs-0.28/src/s__map.cc <==
/*$Id: s__map.cc,v 17.1 2000/02/19 08:51:13 al Exp $ -*- C++ -*-
 * Allocates space for the admittance matrix.  Allocation below the
 * diagonal is by row, above the diagonal is by column, and stored backwards.
 * This broken vector is stored.  The length of it increases with increasing
 * position.  The maximum length of the nth vector is 2n-1.  For a band matrix
 * only those elements that are non-zero or are nearer to the diagonal than a
 * non-zero element are stored.
 */
#include "e_card.h"
#include "u_status.h"
#include "s__.h"
/*--------------------------------------------------------------------------*/
//static void	 SIM::map_nodes();
//static void	 SIM::order_reverse();
//static void	 SIM::order_forward();
//static void	 SIM::order_auto();
//static void	 SIM::map_list(CARD*);
/*--------------------------------------------------------------------------*/
/* map_nodes: map intermediate node number to internal node number.
 * Ideally, this function would find some near-optimal order
 * and squash out gaps.
 */
/*static*/ void SIM::map_nodes()
{
  nm = new int[STATUS::total_nodes+1];
  STATUS::order.reset().start();
  switch (OPT::order){
    default:
      error(bWARNING, "invalid order spec: %d\n", OPT::order);
    case oAUTO:	   order_auto();    break;
    case oREVERSE: order_reverse(); break;
    case oFORWARD: order_forward(); break;
  }
  STATUS::order.stop();
  map_list(CARD_LIST::card_list);
}
/*--------------------------------------------------------------------------*/
/* order_reverse: force ordering to reverse of user ordering
 *  subcircuits at beginning, results on border at the bottom
 */
/*static*/ void SIM::order_reverse()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = STATUS::total_nodes - node + 1;
}
/*--------------------------------------------------------------------------*/
/* order_forward: use user ordering, with subcircuits added to end
 * results in border at the top (worst possible if lots of subcircuits)
 */
/*static*/ void SIM::order_forward()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = node;
}
/*--------------------------------------------------------------------------*/
/* order_auto: full automatic ordering
 * reverse, for now
 */
/*static*/ void SIM::order_auto()
{
  nm[0] = 0;
  for (int node = 1;  node <= STATUS::total_nodes;  ++node)
    nm[node] = STATUS::total_nodes - node + 1;
}
/*--------------------------------------------------------------------------*/
/* map_list: map intermediate node to internal (working) node number.
 * recursive: actual mapping done here
 */
/*static*/ void SIM::map_list(CARD_LIST& cl)
{
  for (CARD_LIST::iterator  ci = cl.begin();  ci != cl.end();  ++ci){
    CARD* brh = *ci;
    if (brh->is_device()){
      int ii;
      for (ii = 0;  brh->_n[ii].t != INVALIDNODE;  ii++){
	brh->_n[ii].m = nm[brh->_n[ii].t];
      }
      brh->_n[ii].m = INVALIDNODE;
      if (brh->subckt().exists()){
	map_list(brh->subckt());
      }
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
